from utils_ak.interactive_imports import *
from qset_feature_store.featureset_engine.dataset_manager.dataset_manager import DatasetManager


class LocalDatasetManager(DatasetManager):
    def __init__(self, data_path):
        self.data_path = data_path
        makedirs(self.data_path)

    def _fn(self, dataset):
        base_fn = str(dataset['_id'])
        return os.path.join(self.data_path, 'local', base_fn, base_fn + '.parquet')

    def get_from_dataset(self, dataset):
        if not os.path.exists(self._fn(dataset)):
            raise KeyError('Local configured featureset not found')
        # todo: make index properly
        df = pd_read(self._fn(dataset))
        return df

    def put_dataset(self, df, dataset, **kwargs):
        makedirs(self._fn(dataset))
        pd_write(df, self._fn(dataset))


def test_local_dataset_manager():
    from qset_feature_store.repository.client import QsetRepositoryClient
    cli = QsetRepositoryClient('akadaner', db='feature-store-test')

    for key in ['featureset', 'dataset_schema', 'dataset']:
        for fs in cli.get_many(key):
            cli.delete(key, fs)

    fs = cli.create_featureset('my_featureset', engine='pandas/v1', body={'code': 'print("Hello World")'})
    fs = cli.push('featureset', fs)

    dss = cli.create_dataset_schema('my_dataset_schema', fs, params={'param': 'foo'})
    dss = cli.push('dataset_schema', dss)

    ds = cli.create_dataset('my_dataset', {'universe': 'binance'}, dss, materialization={'storage': 'local'})
    ds = cli.push('dataset', ds)

    ldm = LocalDatasetManager('data/')

    df = pd.DataFrame([1, 2, 3], columns=['a'])

    ldm.put(df, ds)

    print(ldm.get(ds))


if __name__ == '__main__':
    test_local_dataset_manager()



from utils_ak.interactive_imports import *
from qset_feature_store.interactive_imports import *


configure_logging(stream=True, stream_level=logging.DEBUG)


class StreamingServer(ProductionMicroservice):
    def __init__(self):
        super().__init__()
        self.engines = []
        self.add_timer(self.process_engines, 1.0)  # try to close windows every one second
        self.add_callback('datasets', 'streaming', self.on_dataset)
        self.add_callback('feed', '', self.on_dataset_feed)

    def on_dataset(self, topic, msg):
        print('On dataset', topic, msg)
        dataset = msg
        engine = StreamingEngine(dataset)
        self.engines.append(engine)

    def on_dataset_feed(self, topic, msg):
        print('adding', msg['ts'], msg)
        for engine in self.engines:
            engine.add(msg['ts'], msg)

    def process_engines(self):
        # print('Processing engines', len(self.engines))
        for engine in self.engines:
            engine.process()


class Ping(ProductionMicroservice):
    def __init__(self, *args, **kwargs):
        super().__init__('Ping microservice', *args, **kwargs)

    def send_ping(self, topic, msg):
        self.logger.info(f'Received {topic} {msg}')
        cur_dt = cast_dt('2020.10.01 00:00:00')

        # time.sleep(3)
        # self.publish_json('datasets', 'streaming', get_sample_streaming_dataset())

        time.sleep(3)

        while True:
            self.publish_json('feed', '', {'ts': str(cur_dt), 'dataset': 'timebars', 'row': {'ticker': 'XTZ-USDT', 'pHigh': random.uniform(2.2, 2.4), 'pLow': random.uniform(2.0, 2.2)}})
            self.publish_json('feed', '', {'ts': str(cur_dt), 'dataset': 'timebars', 'row': {'ticker': 'BTC-USDT', 'pHigh': random.uniform(2.2, 2.4), 'pLow': random.uniform(2.0, 2.2)}})
            time.sleep(3)
            cur_dt += timedelta(minutes=10)


def run_ping():
    ping = Ping()

    async def send_initial():
        await asyncio.sleep(1.0)
        ping.send_ping('init', 'init')

    ping.tasks.append(asyncio.ensure_future(send_initial()))
    ping.run()


if __name__ == '__main__':
    multiprocessing.Process(target=run_ping).start()
    server = StreamingServer()
    # run_listener_async('heartbeat', brokers_config=server.brokers_config)
    server.run()




from qset_feature_store.archive.data_provider import *
from .featureset import *
from .reflexion import *
from qset_feature_store.dataset_manager.dataset_manager import *
from .repository import *
from .dataset_manager import *


import fire
from functools import partial
from utils_ak.mongo_job_queue.worker.worker import *
from qset_feature_store.worker.dataset.dataset_worker import DatasetWorker

if __name__ == '__main__':
    fire.Fire(partial(run_worker, worker_cls=DatasetWorker))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qset_feature_store',
 'qset_feature_store.archive',
 'qset_feature_store.archive.dataset_manager',
 'qset_feature_store.archive.worker',
 'qset_feature_store.archive.worker.dataset',
 'qset_feature_store.controller',
 'qset_feature_store.featureset_engine',
 'qset_feature_store.featureset_engine.dataset_manager',
 'qset_feature_store.repository']

package_data = \
{'': ['*'],
 'qset_feature_store.featureset_engine': ['dataset_manager/data/local/6026716f1ae81052b43a9818/*',
                                          'dataset_manager/data/local/6026721a87f650a55707dfdb/*',
                                          'dataset_manager/data/local/60267e93e653a081d4e5456e/*',
                                          'dataset_manager/data/local/602a170a9d5e0dd1173f7956/*',
                                          'dataset_manager/data/local/602a17284e0f5bc2e64031d2/*',
                                          'dataset_manager/data/local/602a1749ec11cc0d869773dd/*',
                                          'dataset_manager/data/local/602a18b5d359dc90e893dddf/*',
                                          'dataset_manager/data/local/602a19077cd15c8858ac48dd/*',
                                          'dataset_manager/data/local/602a190c82e20d0e7896a40e/*',
                                          'dataset_manager/data/local/602a1c22827b0e4bd50a0d92/*',
                                          'dataset_manager/data/local/602a1c540626c6fcb56b0f82/*',
                                          'dataset_manager/data/local/602a1c6c53e6aa1620d4fe38/*',
                                          'dataset_manager/data/local/602a1c78c07ce140072ce287/*',
                                          'dataset_manager/data/local/602a1c8de4f2329deae0ab9c/*',
                                          'dataset_manager/data/local/602a1c9ad74fc7857d22b316/*',
                                          'dataset_manager/data/local/602a1cdef42338b9b07816c7/*',
                                          'dataset_manager/data/local/602a1cf2ca38d994f8054fde/*',
                                          'dataset_manager/data/local/602a1d08b73b600289fb0384/*',
                                          'dataset_manager/data/local/602a1d1ee401c348f08eb2c6/*',
                                          'dataset_manager/data/local/602a1d25b63b29352f27bb98/*',
                                          'dataset_manager/data/local/602a1eff8b062bf201d65e9b/*',
                                          'dataset_manager/data/local/602a2fdfff1a6476d0374458/*',
                                          'dataset_manager/data/local/602a2fe6987698907accae8e/*',
                                          'dataset_manager/data/local/binance/6024fd64f2ca39cea598e399/*',
                                          'dataset_manager/data/local/binance/602512fc6b2eb3031fa883b8/*',
                                          'dataset_manager/data/local/binance/6025152e76442bb46c82a129/*',
                                          'dataset_manager/data/local/binance/602524ad96fa8f4c4ea94dbf/*',
                                          'dataset_manager/data/local/binance/602524d1dd67effe309c2a25/*',
                                          'dataset_manager/data/local/binance/602524e36f32a6ab7bfb526d/*',
                                          'dataset_manager/data/local/binance/6025255f76b11d190d87b624/*',
                                          'dataset_manager/data/local/binance/60252577681f7b4e87e72847/*',
                                          'dataset_manager/data/local/binance/60252584db4dad5a6c2cdbb3/*',
                                          'dataset_manager/data/local/binance/6025258b3261851e6fb1f6bb/*',
                                          'dataset_manager/data/local/binance/602525a6b179830bebd173ba/*',
                                          'dataset_manager/data/local/binance/602525ade235cbb1ed304760/*',
                                          'dataset_manager/data/local/binance/60267f50ba8ff2704b3a859a/*',
                                          'dataset_manager/data/local/binance/60267f8f7f46bb76f2133f66/*',
                                          'dataset_manager/data/local/binance/60267fe33a3bbc099730f023/*',
                                          'dataset_manager/data/local/binance/60268009ef38f9d2bdb48c76/*',
                                          'dataset_manager/data/local/binance/6026805c0a38520ab0bd61de/*',
                                          'dataset_manager/data/local/binance/602680e0e88e185a147ce393/*',
                                          'dataset_manager/data/local/binance/602680fd1b846353f619ffca/*',
                                          'dataset_manager/data/local/binance/6026813ccb5670a4bbb50523/*',
                                          'dataset_manager/data/local/binance/602681524ddf3907c5c95877/*',
                                          'dataset_manager/data/local/binance/6026817cfb1aae4e22b7b34b/*',
                                          'dataset_manager/data/local/binance/602681857d8b83e4a26e4af7/*',
                                          'dataset_manager/data/local/default/60267fbaaf52325147eafffa/*']}

install_requires = \
['utils-ak>=0.1.6,<0.2.0']

setup_kwargs = {
    'name': 'test-qset-feature-store',
    'version': '0.1.3',
    'description': '',
    'long_description': None,
    'author': 'Arsenii Kadaner',
    'author_email': 'arseniikadaner@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.1,<4.0.0',
}


setup(**setup_kwargs)
